import * as path from "path";
import * as repoAction from "./RepoAction";
import { CleanOptions } from "./utils/Git";
import { Dependency, loadYml, writeYml } from "./YmlLoader";
import { relativePath } from "./utils/FileHandler";
import chalk from "chalk";
import { loadFlattedDep } from "./RepoLoader";

/**
 * 依次从依赖树左侧节点读取内容，会导致后一个节点的子节点也覆盖顶层内容，需要用 flat 方法，重新操作内容。
 * @param repoDir yml 所在的目录
 * @param action 对 yml 依赖库所采取的操作, {config: 从 repoDir 中读取出来的依赖信息，repoDir：yml 文件所在的目录，与 repoDir 参数一致，options：部分 action 函数需要的额外参数，与 repoDir 一致}
 * @param options 部分操作的额外参数
 */
async function recursiveAction(repoDir: string, action: (config: Dependency, repoDir?: string, options?: any) => void | Promise<any>, options?: any): Promise<void> {
    const dependencies = loadYml(repoDir);
    for (let dependency of dependencies) {
        let subDir = path.resolve(repoDir, dependency.repoPath);
        await action(dependency, repoDir, options);
        const nestedRepos = loadYml(subDir);
        if (nestedRepos.length > 0) {
            await recursiveAction(subDir, action, options);
        }
    }
}

export async function clone(repoDir: string): Promise<void> {
    await recursiveAction(repoDir, repoAction.clone);
    const flattedDependencies = loadFlattedDep(repoDir);
    for (let dependency of flattedDependencies) {
        await repoAction.clone(dependency, dependency.parent);
    }
}

export async function pull(repoDir: string): Promise<void> {
    await recursiveAction(repoDir, repoAction.pull);
    const flattedDependencies = loadFlattedDep(repoDir);
    for (let dependency of flattedDependencies) {
        await repoAction.pull(dependency, dependency.parent);
    }
}

export async function checkout(repoDir: string): Promise<void> {
    const flattedDependencies = loadFlattedDep(repoDir);
    for (let dependency of flattedDependencies) {
        await repoAction.checkout(dependency, dependency.parent);
    }
}

export async function checkCommit(repoDir: string): Promise<void> {
    const flattedDependencies = loadFlattedDep(repoDir);
    for (let dependency of flattedDependencies) {
        await repoAction.checkCommit(dependency, dependency.parent);
    }
}

export async function state(repoDir: string): Promise<void> {
    await recursiveAction(repoDir, repoAction.state);
}

export async function update(repoDir: string, repo?: string): Promise<void> {
    const repos = loadYml(repoDir);
    let oldCommits = repos.map(c => c.commit);

    const p = async (config: Dependency) => {
        await repoAction.update(config, repoDir);
        let subRepo = path.resolve(repoDir, config.repoPath);
        await update(subRepo);
    }
    const promiseMap = repos.filter(v => v.name === repo || repo === undefined).map(v => p(v));
    await Promise.all(promiseMap);

    await checkModify(repoDir);

    const diff = repos.map((config, i) => {
        return { ...config, old: oldCommits[i] };
    }).filter((config) => {
        return config.commit !== config.old;
    }).map(config => {
        return `repo:${config.name} in ${relativePath(repoDir)} yml will change commit from ${chalk.green(config.old ? config.old : "no commit")} to new commit ${chalk.green(config.commit!)}`;
    });

    if (diff.length !== 0) {
        const str = diff.reduce(v => { return "\n" + v });
        console.log(`${relativePath(repoDir)} yml will update\n${str}`);
        writeYml(repoDir, repos);
    }
}

export async function checkModify(repoDir: string): Promise<void> {
    const flattedDependencies = loadFlattedDep(repoDir);
    for (let dependency of flattedDependencies) {
        await repoAction.checkModify(dependency, dependency.parent);
    }
}

export async function stash(repoDir: string): Promise<void> {
    const flattedDependencies = loadFlattedDep(repoDir);
    for (let dependency of flattedDependencies) {
        await repoAction.stash(dependency, dependency.parent);
    }
}

export async function clean(repoDir: string, options: CleanOptions): Promise<void> {
    const flattedDependencies = loadFlattedDep(repoDir);
    for (let dependency of flattedDependencies) {
        await repoAction.clean(dependency, dependency.parent, options);
    }
}