import * as path from "path";
import { getPathState, FileState, relativePath } from "./utils/FileHandler";
import { Dependency } from "./YmlLoader";
import * as git from "./utils/Git";
import chalk from "chalk";


export async function pull(config: Dependency, dir: string): Promise<void> {
    const repoDir = path.resolve(dir, config.repoPath);
    const repoState = await getPathState(repoDir);

    switch (repoState) {
        case FileState.None:
            await git.clone(path.resolve(repoDir, ".."), config.repoURL, path.parse(repoDir).base);
            break;
        case FileState.Dir:
            await git.pull(dir, config.repoURL);
            break;
        case FileState.File:
        default:
            throw new Error(`pull: ${repoDir} is ${repoState}`);
    }
}

export async function clone(config: Dependency, dir: string): Promise<void> {
    const repoDir = path.resolve(dir, config.repoPath);
    const repoState = await getPathState(repoDir);

    switch (repoState) {
        case FileState.None:
        {
            await git.clone(path.resolve(repoDir, ".."), config.repoURL, path.parse(config.repoPath).base);
            if (config.commit) {
                await git.reset(repoDir, config.commit!);
            }
            break;
        }
        case FileState.Dir:
        {
            const currentCommit = git.readCommit(repoDir, config.commit ? config.commit.length : undefined);
            if (currentCommit == config.commit) {
                console.log(`${config.name} in ${relativePath(repoDir)} do not need do any thing`);
            } else {
                console.log(`${config.name} in ${path.basename(dir)}:${relativePath(repoDir)} already exists, begin fetch from: ${config.repoURL} and reset to ${chalk.green(config.commit!)}`);
                await git.reset(repoDir, config.commit!);
            }
            break;
        }
        case FileState.File:
        default:
            throw new Error(`clone: ${repoDir} is ${repoState}`);
    }
}

export async function clean(config: Dependency, dir: string, options: git.CleanOptions): Promise<void> {
    const repoDir = path.resolve(dir, config.repoPath);
    const repoState = await getPathState(repoDir);

    switch (repoState) {
        case FileState.None:
            console.log(`clean: no such directory: ${repoDir}`);
            break;
        case FileState.Dir:
            await git.clean(dir, options);
            break;
        case FileState.File:
        default:
            throw new Error(`pull: ${repoDir} is ${repoState}`);
    }
}

export async function checkModify(config: Dependency, dir: string): Promise<void> {
    const repoDir = path.resolve(dir, config.repoPath);
    const repoState = await getPathState(repoDir);
    switch (repoState) {
        case FileState.None:
            console.log(`checkModify: no such directory: ${repoDir}`);
            break;
        case FileState.Dir:
        {
            const status = git.status(repoDir, "s");
            if (status.length > 0) {
                const something = git.status(repoDir);
                throw new Error(`repo:${config.name} in ${relativePath(repoDir)} yml: need something to resolve first:\n${something}`);
            }
            break;
        }
        case FileState.File:
        default:
            throw new Error(`checkModify: ${repoDir} is ${repoState}`);
    }
}

export async function checkout(config: Dependency, dir: string): Promise<void> {
    const repoDir = path.resolve(dir, config.repoPath);
    if (!config.commit) {
        throw new Error(`${repoDir} has no commit config`);
    }
    const repoState = await getPathState(repoDir);
    switch (repoState) {
        case FileState.Dir:
        {
            git.checkout(repoDir, config.commit);
            break;
        }
        case FileState.None:
            throw new Error(`checkout: no such directory: ${repoDir}`);
        case FileState.File:
        default:
            throw new Error(`checkout: ${repoDir} is ${repoState}`);
    }
}

export async function checkCommit(config: Dependency, dir: string): Promise<void> {
    const repoDir = path.resolve(dir, config.repoPath);
    const repoState = await getPathState(repoDir);

    switch (repoState) {
        case FileState.Dir:
        {
            //readCommit 的长度与 yml commit 记录位数可能不一致
            const repoCommit = git.readCommit(repoDir, config.commit ? config.commit.length : undefined);
            if (repoCommit === config.commit) {
                console.log(`repo: ${config.name} in ${relativePath(repoDir)}'s commit-id is ${repoCommit} ${chalk.green("[match]")} ${relativePath(dir)}'s yml commit-id`);
            } else {
                throw new Error(`repo: ${config.name} in ${relativePath(repoDir)}'s commit-id is ${chalk.red(repoCommit)} ${chalk.red("[mismatch]")} ${relativePath(dir)}'s yml commit-id which is ${config.commit}`);
            }
            break;
        }
        case FileState.None:
            throw new Error(`checkCommit: no such directory: ${repoDir}`);
        case FileState.File:
        default:
            throw new Error(`checkCommit: ${repoDir} is ${repoState}`);
    }
}

export async function state(config: Dependency, dir: string): Promise<void> {
    const repoDir = path.resolve(dir, config.repoPath);
    const repoState = await getPathState(repoDir);

    switch (repoState) {
        case FileState.Dir:
        {
            const repoCommit = git.readCommit(repoDir, config.commit ? config.commit.length : undefined);
            if (repoCommit === config.commit) {
                console.log(`repo: ${config.name} in ${relativePath(repoDir)}'s commit-id is ${repoCommit} ${chalk.green("[match]")} ${relativePath(dir)}'s yml commit-id`);
            } else {
                console.log(`repo: ${config.name} in ${relativePath(repoDir)}'s commit-id is ${chalk.red(repoCommit)} ${chalk.red("[mismatch]")} ${relativePath(dir)}'s yml commit-id which is ${config.commit}`);
            }
            break;
        }
        case FileState.None:
            throw new Error(`state: no such directory: ${repoDir}`);
        case FileState.File:
        default:
            throw new Error(`state: ${repoDir} is ${repoState}`);
    }
}

export async function update(config: Dependency, dir: string): Promise<void> {
    const repoDir = path.resolve(dir, config.repoPath);
    const repoState = await getPathState(repoDir);
    switch (repoState) {
        case FileState.Dir:
        {
            config.commit = git.readCommit(repoDir, config.commit ? config.commit.length : undefined);
            break;
        }
        case FileState.None:
            throw new Error(`update: no such directory: ${repoDir}`);
        default:
            throw new Error(`state: ${repoDir} is ${repoState}`);
    }
}

export async function stash(config: Dependency, dir: string): Promise<void> {
    const repoDir = path.resolve(dir, config.repoPath);
    const repoState = await getPathState(repoDir);

    switch (repoState) {
        case FileState.Dir:
        {
            const status = git.status(repoDir, "s");
            if (status.length > 0) {
                console.log(chalk.green(`status ${repoDir}: will stash modify`));
                await git.stash(repoDir);
            }
            break;
        }
        case FileState.None:
            console.log(`stash: no such directory: ${repoDir}`);
        default:
            throw new Error(`state: ${repoDir} is ${repoState}`);
    }
}