import * as path from "path";
import { readRemote } from "./utils/Git";
import { Dependency, loadYml } from "./YmlLoader";

export type ResDescription = Dependency & {
    /** 当前节点所在的目录 */
    dir: string;
    /** 当前资源的父节点所在的目录，可以与 Dependency 中的 repoPath 结合，直接给以前的 API 使用 */
    parent: string;
    /** 子节点 */
    children: ResDescription[];
}

export function loadRepo(dir: string): ResDescription {
    const rootYml = rootRepo(dir);
    const dependencies = loadYml(dir);
    rootYml.children = loadChildren(dir, dependencies);
    return rootYml;
}

function rootRepo(dir: string): ResDescription {
    const name = path.parse(dir).name;
    return {
        name,
        dir,
        repoPath: ".",
        repoURL: readRemote(dir),
        parent: "root",
        children: [],
    };
}

function loadChildren(dir: string, deps: Dependency[]): ResDescription[] {
    const children = deps.map(d => {
        const ymlDir = path.resolve(dir, d.repoPath);
        const dependencies = loadYml(ymlDir);
        const nestedChildren = loadChildren(ymlDir, dependencies);
        return {
            ...d,
            dir: ymlDir,
            parent: dir,
            children: nestedChildren,
        };
    });
    return children;
}

export function loadFlattedDep(dir: string): ResDescription[] {
    const res = loadRepo(dir);
    return flatDependencies(res);
}

/**
 * 树状结构展开为一维数组。
 * 当 dir 字段内容相同时，上层覆盖下层；同一层，右边覆盖左边的内容。
 * 返回的内容，从顶端左上角返回
 * @param yml 
 */
export function flatDependencies(yml: ResDescription): ResDescription[] {
    const nestedSub = yml.children.map(sub => {
        return flatDependencies(sub);
    });
    return deduplicate([ ...yml.children, ...flatTwoDimensionalArray(nestedSub, "dir")], "dir",);
}

/**
 * 将二维数组转化为一维数组，同时根据 key 字段进行去重处理。
 * @param two 二维数组
 * @param key 去重关键字
 * @param reverse 选择重复时，保留的元素。true 时，保留后面的元素；false 时，保留前面元素。默认为 false，保留前面的元素。（与 deduplicate 一致）
 */
function flatTwoDimensionalArray<T, K extends keyof T>(two: T[][], key: K, reverse = true): T[] {
    return two.reduce((p, c) => {
        return reverse ? deduplicate([...p, ...c].reverse(), key) : deduplicate([...p, ...c], key);
    }, []);
}

/**
 * 根据 key ，对 list 进行去重处理。如果 key 字段相同，会保留前面。
 * @param list 数组
 * @param key 去重关键字
 */
function deduplicate<T, K extends keyof T>(list: T[], key: K): T[] {
    const keys = new Set();
    return list.filter(v => {
        const contains = keys.has(v[key]);
        keys.add(v[key]);
        return !contains;
    });
}