import * as fs from "fs";
import * as path from "path";
import { readCommit, readRemote } from "./utils/Git";
import { WorkDir, relativePath, getPathState, FileState } from "./utils/FileHandler";
const yaml = require("js-yaml");

export type Dependency = RepoInfo & Lock;

const resourceContent: RepoInfo = {
    name: 'string',
    repoPath: 'string',
    repoURL: 'string',
}

type RepoInfo = {
    name: string,
    /** clone 的相对路径，由于历史原因，不改动字段名 */
    repoPath: string,
    repoURL: string,
}

const lockContent: Lock = {
    name: 'string',
    commit: 'string',
}

type Lock = {
    name: string,
    commit?: string,
}

const carrot_yml = "carrot.yml";
const lock_yml = "lock.yml";

export function findYmlDir(dir = WorkDir): string {
    if (getPathState(path.resolve(dir, carrot_yml)) === FileState.File) {
        console.log(`find yml in: ${relativePath(dir)}`);
        return dir;
    }
    if (dir === "/") {
        throw new Error(`no ${carrot_yml} exist or any of the parent directories`);
    }
    const parentDir = path.resolve(dir, "..");
    return findYmlDir(parentDir);
}

export function writeYml(dir: string, data: Dependency[]): void {

    if (!data || data.length == 0) {
        return;
    }

    dir = parseLegacyYmlPath(dir);
    fs.writeFileSync(path.resolve(dir, carrot_yml), yaml.dump(data));
    // TODO: 资源与 commit 锁分开存储
}

export function loadYml(dir: string): Dependency[] {
    dir = parseLegacyYmlPath(dir);
    let carrots = loadCarrot(dir);
    let locks = loadLock(dir);
    return mergeConfig(carrots, locks);
}

export function initYml(dir = WorkDir): void {
    const data = loadYml(dir);
    for (let subConfig of data) {
        const commit = readCommit(path.resolve(dir, subConfig.repoPath));
        const repoURL = readRemote(path.resolve(dir, subConfig.repoPath))
        subConfig.commit = commit;
        subConfig.repoURL = repoURL;
    }
    writeYml(WorkDir, data);
}

function parseLegacyYmlPath(dir: string): string {
    if (path.basename(dir) === carrot_yml) {
        dir = path.dirname(dir);
    }
    return dir;
}

function loadCarrot(dir: string): RepoInfo[] {
    const carrotPath = path.resolve(dir, carrot_yml);
    if (getPathState(carrotPath) === FileState.File) {
        return yaml.safeLoad(fs.readFileSync(carrotPath, 'utf-8'));
    } else {
        return [];
    }
}

function loadLock(dir: string): Lock[] {
    const lockPath = path.resolve(dir, lock_yml);
    if (getPathState(lockPath) === FileState.File) {
        return yaml.safeLoad(fs.readFileSync(lockPath, 'utf-8'));
    } else {
        return [];
    }
}

function mergeConfig(resources: RepoInfo[], locks: Lock[]): Dependency[] {
    return resources.map(r => Object.assign(r, locks.find(l => l.name === r.name))) as Dependency[];
}

function splitConfig(module: Dependency): { resource: RepoInfo, lock: Lock } {

    let resource = {} as RepoInfo;
    let lock = {} as Lock;

    for (const key in module) {
        if (module.hasOwnProperty(key)) {
            const value = (<any>module)[key];

            if (keyInType<RepoInfo, keyof RepoInfo>(key, resourceContent)) {
                resource[key] = value;
            }
            if (keyInType<Lock, keyof Lock>(key, lockContent)) {
                lock[key] = value;
            }
        }
    }
    return { resource, lock };
}

// TODO: 强约束 x:string 。但是这样要增加对 T 的约束，保证 keyof T 出来的是 string，暂时不会写
function keyInType<T, K extends keyof T>(x: any, instance: T): x is K {
    return x in instance;
}