import "source-map-support/register";
import * as action from "./Action";
import * as yml from "./YmlLoader";
import chalk from 'chalk';
import { loadRepo, flatDependencies } from "./RepoLoader";

type Argv = {
    verbose: boolean;
    hard: boolean;
    repo: string;
}

type PrivateArgv = Argv & {
    path: string;
}

export async function load(argv: Argv): Promise<void>
{
    try {
        const ymlDir = yml.findYmlDir();
        const ymlRepo = loadRepo(ymlDir);
        console.log(ymlRepo);
    } catch (error) {
        if (argv.verbose) {
            console.log(chalk.red(JSON.stringify(error)));
        } else {
            console.log(chalk.red(error.message));
        }
        process.exit(1);
    }
}

export function flatDep(argv: Argv): void {
    try {
        const ymlDir = yml.findYmlDir();
        const ymlRepo = loadRepo(ymlDir);
        const flat = flatDependencies(ymlRepo);
        console.log(flat);
    } catch (error) {
        if (argv.verbose) {
            console.log(chalk.red(JSON.stringify(error)));
        } else {
            console.log(chalk.red(error.message));
        }
        process.exit(1);
    }
}

export async function clone(argv: Argv): Promise<any> {
    try {
        const ymlDir = yml.findYmlDir();
        if (argv.hard) {
            await stash(argv);
        } else {
            await checkModify({...argv, path: ymlDir});
        }

        await action.clone(ymlDir);
    } catch (error) {
        if (argv.verbose) {
            console.log(chalk.red(JSON.stringify(error)));
        } else {
            console.log(chalk.red(error.message));
        }
        process.exit(1);
    }
}

export async function checkout(argv: Argv): Promise<void> {
    try {
        const ymlDir = yml.findYmlDir();
        await checkModify({...argv, path: ymlDir});
        await action.checkout(ymlDir);
    } catch (error) {
        if (argv.verbose) {
            console.log(chalk.red(JSON.stringify(error)));
        } else {
            console.log(chalk.red(error.message));
        }        
        process.exit(1);
    }
}

export async function update(argv: Argv): Promise<void> {
    try {
        const ymlDir = yml.findYmlDir();
        await checkModify({...argv, path: ymlDir});
        await action.update(ymlDir, argv.repo);
    } catch (error) {
        if (argv.verbose) {
            console.log(chalk.red(JSON.stringify(error)));
        } else {
            console.log(chalk.red(error.message));
        }        
        process.exit(1);
    }
}

export async function pull(argv: Argv): Promise<void> {
    try {
        const ymlDir = yml.findYmlDir();
        await checkModify({...argv, path: ymlDir});
        await action.pull(ymlDir);
    } catch (error) {
        if (argv.verbose) {
            console.log(chalk.red(JSON.stringify(error)));
        } else {
            console.log(chalk.red(error.message));
        }
        process.exit(1);
    }
}

export async function state(argv: Argv): Promise<void> {
    try {
        const ymlDir = yml.findYmlDir();
        await action.state(ymlDir);
    } catch (error) {
        if (argv.verbose) {
            console.log(chalk.red(JSON.stringify(error)));
        } else {
            console.log(chalk.red(error.message));
        }
        process.exit(1);
    }
}

export async function check(argv: Argv): Promise<void> {
    try {
        const ymlDir = yml.findYmlDir();
        await checkModify({...argv, path: ymlDir});
        await action.checkCommit(ymlDir);
        console.log(chalk.green(`all dependences pass`));
    } catch (error) {
        if (argv.verbose) {
            console.log(chalk.red(JSON.stringify(error)));
        } else {
            console.log(chalk.red(error.message));
        }
        process.exit(1);
    }
}

export async function stash(argv: Argv): Promise<void> {
    try {
        const ymlDir = yml.findYmlDir();
        await action.stash(ymlDir);
    } catch (error) {
        if (argv.verbose) {
            console.log(chalk.red(JSON.stringify(error)));
        } else {
            console.log(chalk.red(error.message));
        }
        process.exit(1);
    }
}

async function checkModify(argv: PrivateArgv): Promise<void> {
    try {
        await action.checkModify(argv.path);
    } catch (error) {
        if (argv.verbose) {
            console.log(chalk.red(JSON.stringify(error)));
        } else {
            console.log(chalk.red(error.message));
        }
        process.exit(1);
    }
}

export async function clean(argv: Argv): Promise<void> {
    const {verbose, repo, hard, ...resetOptions} = argv;
    try {
        const ymlDir = yml.findYmlDir();
        await action.clean(ymlDir, resetOptions);
    } catch (error) {
        if (argv.verbose) {
            console.log(chalk.red(JSON.stringify(error)));
        } else {
            console.log(chalk.red(error.message));
        }
        process.exit(1);
    }
}