import { execSyncInDir, execInDir } from "./Shell";

export function readRemote(dir: string): string {
    const names = execSyncInDir(dir, "git remote").split("\n");
    const name = names.find(n => n.startsWith("origin")) || names[0];
    return execSyncInDir(dir, `git remote get-url ${name}`);
}

export function readCommit(dir: string, length?: number): string {
    if (length) {
        return execSyncInDir(dir, `git rev-parse --short=${length} HEAD`);
    }
    return execSyncInDir(dir, `git rev-parse --short HEAD`);
}

export async function clone(rootDir: string, url: string, name: string): Promise<void> {
    await execInDir(rootDir, `git clone --progress ${url} ${name}`);
}

export async function pull(dir: string, url: string): Promise<void> {
    await execInDir(dir, `git pull --ff-only ${url} master --allow-unrelated-histories`);
}

export async function reset(dir: string, commit: string): Promise<void> {
    execSyncInDir(dir, `git fetch --all && git reset --hard ${commit}`);
}

export function checkout(dir: string, commit: string): void {
    execSyncInDir(dir, `git checkout ${commit}`);
}

export async function stash(dir: string): Promise<void> {
    const currentCommit = readCommit(dir);
    await execInDir(dir, `git stash push -m "script stash at ${currentCommit}" --include-untracked`);
}

export function status(dir: string, options?: string): string {
    const cmd = options === undefined ? `git status` : `git status -${options}`;
    return execSyncInDir(dir, cmd);
}

export type CleanOptions = Partial<{
    e: string;
    d: boolean;
    f: boolean;
    i: boolean;
    n: boolean;
    q: boolean;
    x: boolean;
    X: boolean;
}>

export async function clean(dir: string, cleanOptions: CleanOptions): Promise<String> {
    let options = ``;

    let key: keyof CleanOptions;
    for (key in cleanOptions) {
        if (cleanOptions.hasOwnProperty(key)) {
            const value = cleanOptions[key];
            if (typeof value === "boolean") {
                options = ` -${key} ${options}`
            } else if (typeof value === "string") {
                options = ` -${key} ${value} ${options}`;
            } else {
                throw new Error("clean options has unexcept option type: " + value);
            }
        }
    }

    return execInDir(dir, `git clean ${options}`);
}